%
% Resolution example.  This script computes the regularized solution for 
% alpha=0.1, estimates the diagonal of the resolution matrix, and then 
% compares the estimate against accurately computed values in 100 randomly
% chosen columns.
%
% Reset everything so that we get repeatable results.
%
% First, clear memory and reset the RNG's, so we get consistent results.
%
clear
rand('state',0);
randn('state',0);
%
% Load in the problem data.
%
load example.mat
%
% Globals.
% 
global GT;
global G;
global L;
global LT;
global alpha;
%
% Calculate some stuff not stored in the file.
%
GT=G';
LT=L';
%
% Some size parameters.
%
[m,n]=size(G);
[r,s]=size(L);
%
% Pad d with extra 0's.
%
dpad=sparse([d; zeros(r,1)]);
%
% Set the regularization parameter.
%
alpha=0.1;
%
% Find the regularized solution.
%
mreg=lsqr(@Gregmult,dpad,[],1500);
%
% Output some measures of the result.
%
fprintf(1,'norm(d) is %f \n',norm(d));
fprintf(1,'norm(G*m-d) is %f \n',norm(G*mreg-d));
fprintf(1,'norm(alpha*L*m) is %f \n',norm(alpha*(L*mreg)));
%
% Now, repeat the process of estimating the diagonal ntimes.
%
ntimes=20;
diags=zeros(ntimes,n);
for k=1:ntimes
%
% In this case, we'll just use 256 random vectors.
%
  numcols=256;
  fprintf('Starting diag estimation %d\n',k);
%
% Estimate the diagonal of R.
%
  diagR=diagestrand(@Rmult,n,numcols);
%
% Store the estimate.  
%
  diags(k,:)=diagR';
end
%
% Find the mean of the estimates.
%
diagR=median(diags);
%
% Now, check a random collection of columns to see how well we
% did.  Columns of G that are 0 indicate cells that have no ray
% coverage- these are of no interest to us.  
%
ntests=100;
for i=1:ntests
%
% Pick a random column of G.
%
  l=floor(rand*n)+1;
%
% Skip columns that are 0, since the corresponding column of R is also 0.
%
  while (norm(G(:,l))==0)
    l=floor(rand*n)+1;
  end
%
% Finally got a column.  Now, compute this column of R by inverting the spike.
%
  m=zeros(n,1);
  m(l)=1;
  rhs=G*m;
  rhspad=sparse([rhs; zeros(r,1)]);
  ker=lsqr(@Gregmult,rhspad,[],5000);
%
% Now, compare the actual value of R(l,l) to the estimate.
%
  cols(i)=l;
  truevals(i)=ker(l);
  ests(i)=diagR(l);
  relerr(i)=abs(ker(l)-diagR(l))/abs(ker(l));
  abserr(i)=abs(ker(l)-diagR(l));
  fprintf(1,'estimated r(%d)=%.3f, accurate r(%d)=%.3f, relerr=%.2f, abserr=%.3f \n', ...
	  [l; diagR(l); l; ker(l); relerr(i); abserr(i)]);
end
%
% Print out the mean absolute error over the 100 randomly chosen columns.
%
fprintf(1,'The mean absolute error was %f\n',mean(abserr));
%
% Save the results for later analysis.
%
save example01.mat
